/***** alignment.c ************************************************
 * Description: Methods for optimal pairwise sequence alignment.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Thu Dec 16 20:32:47 2004.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <ctype.h>
#include <limits.h>
#include "eprintf.h"
#include "interface.h"
#include "alignment.h"
#include "stringUtil.h"

#define MAXLINE 1000
#define SIZE_CHARSET 256

static int letters[9];
static int alignmentMode = GLOBAL;

void printAlignment(Alignment *al, FILE *fp, SubstitutionMatrix *sm, Args args){
  SeqList *lp1, *lp2;
  char **printout;
  char **head;
  char *number;
  int i, j, l, tc1, tc2, lc1, lc2, start1, start2;

  start1 = al->start1;
  start2 = al->start2;
  printout = (char **)emalloc(3*sizeof(char *));
  head = (char **)emalloc(3*sizeof(char *));
  number = (char *)emalloc(20*sizeof(char));
  for(i=0;i<3;i++){
    printout[i] = (char *)emalloc((args.l+20)*sizeof(char));
    head[i] = (char *)emalloc(20*sizeof(char));
  }
  l = 0;
  tc1 = tc2 = lc1 = lc2 = 0;
  fprintf(fp, "Query:             %s\n",al->name1);
  fprintf(fp, "   Length:         %d\n",al->length1);
  if(alignmentMode == LOCAL)
    fprintf(fp, "   Hit:            %d - %d\n",al->start1,al->end1);
  fprintf(fp, "Subject:           %s\n",al->name2);
  fprintf(fp, "   Length:         %d\n",al->length2);
  if(alignmentMode == LOCAL || alignmentMode == KERROR)
    fprintf(fp, "   Hit:            %d - %d\n",al->start2,al->end2);
  fprintf(fp, "Score:             %.1f\n",al->score);
  if(alignmentMode == KERROR)
    fprintf(fp, "Errors:            %d\n",al->numerr);
  if(al->strand != NULL)
    fprintf(fp, "Strand:            %s\n",al->strand);
  if(!args.a){
    fprintf(fp,"//\n");
    return;
  }else{
    fprintf(fp,"\n\n");
  }
  for(lp1=al->head1, lp2=al->head2; lp1!=NULL && lp2!=NULL; lp1=lp1->next, lp2=lp2->next){
    printout[0][l] = lp1->c;
    printout[2][l] = lp2->c;
    if(isalpha(lp1->c)){
      tc1++;
      lc1++;
    }
    if(isalpha(lp2->c)){
       tc2++;
       lc2++;
    }
    if(printout[0][l] == printout[2][l]){
      if(args.p)
	printout[1][l] = lp1->c;
      else
	printout[1][l] = '|';
    }else if(sm->mat[sm->dic[(int)lp1->c]][sm->dic[(int)lp2->c]] > 0){
      if(isalpha(printout[0][l]) && isalpha(printout[2][l])){
	if(args.p)
	  printout[1][l] = '+';
	else
	  printout[1][l] = '.';
      }else{
	printout[1][l] = ' ';
      }
    }else{
      printout[1][l] = ' ';
    }
    l++;
    if(l == args.l){
      printout[0][l] = printout[1][l] = printout[2][l] = '\0';
      if(tc1 > 0)
	if(lc1 > 0)
	  sprintf(head[0], "Query: %d", start1+tc1-lc1);
	else
	  sprintf(head[0], "Query: %d", start1+tc1-1);
      else
	sprintf(head[0], "Query: 0");
      if(tc2 > 0)
	if(lc2 > 0)
	  sprintf(head[2], "Sbjct: %d", start2+tc2-lc2);
	else
	  sprintf(head[2], "Sbjct: %d", start2+tc2-1);
      else
	sprintf(head[2], "Sbjct: 0");
      j = max(strlen(head[0]), strlen(head[2])) + 1;
      fprintf(fp,"%s", head[0]);
      for(i=strlen(head[0]);i<j;i++)
	fprintf(fp," ");
      fprintf(fp, "%s %d\n", printout[0], start1+tc1-1);
      for(i=0;i<j;i++)
	fprintf(fp," ");
      fprintf(fp, "%s\n", printout[1]);
      fprintf(fp,"%s", head[2]);
      for(i=strlen(head[2]);i<j;i++)
	fprintf(fp," ");
      fprintf(fp, "%s %d\n\n\n", printout[2], start2+tc2-1);
      l = 0;
      lc1 = lc2 = 0;
    }
  }
  if(l > 0){
    printout[0][l] = printout[1][l] = printout[2][l] = '\0';
    if(tc1 > 0)
      if(lc1 > 0)
	sprintf(head[0], "Query: %d", start1+tc1-lc1);
      else
	sprintf(head[0], "Query: %d", start1+tc1-1);
    else
      sprintf(head[0], "Query: 0");
    if(tc2 > 0)
      if(lc2 > 0)
	sprintf(head[2], "Sbjct: %d", start2+tc2-lc2);
      else
	sprintf(head[2], "Sbjct: %d", start2+tc2-1);
    else
      sprintf(head[2], "Sbjct: 0");
    j = max(strlen(head[0]), strlen(head[2])) + 1;
    fprintf(fp,"%s", head[0]);
    for(i=strlen(head[0]);i<j;i++)
      fprintf(fp," ");
    fprintf(fp, "%s %d\n", printout[0], start1+tc1-1);
    for(i=0;i<j;i++)
      fprintf(fp," ");
    fprintf(fp,"%s\n",printout[1]);
    fprintf(fp,"%s", head[2]);
    for(i=strlen(head[2]);i<j;i++)
      fprintf(fp," ");
    fprintf(fp, "%s %d\n", printout[2], start2+tc2-1);
  }
  fprintf(fp,"//\n");
  for(i=0;i<3;i++){
    free(printout[i]);
    free(head[i]);
  }
  free(printout);
  free(head);
  free(number);
}

/* readSubstitutionMatrix: read susbstitution matrix from file;
 * do not distinguish between upper and lower case letters
 */
SubstitutionMatrix *readSubstitutionMatrix(SubstitutionMatrix *sm, FILE *fp){
  char *line;
  char *num, c;
  int i, j, l, n;
  
  /* memory allocation */
  sm = (SubstitutionMatrix *)emalloc(sizeof(SubstitutionMatrix));
  num = (char *)emalloc(5*sizeof(char));
  line = (char *)emalloc(MAXLINE*sizeof(char));
  sm->dic = (int *)emalloc(SIZE_CHARSET*sizeof(int));
  sm->revdic = (char *)emalloc(SIZE_CHARSET*sizeof(char));

  while((line = fgets(line, MAXLINE, fp)) != NULL){
    if(strstr(line,"#") == NULL)
      break;
  }
  /* initialize dictionaries */
  for (i = 0; i < SIZE_CHARSET; i++) {
    sm->dic[i] = 0;
    sm->revdic[i] = 0;
  }
    
  /* discover alphabet from first line of matrix */
  sm->size = 0;
  for(i=0;i<strlen(line);i++){
    c = line[i];
    if(!isspace(c)){
      sm->revdic[sm->size] = c;
      sm->dic[tolower(c)] = sm->size++;
      sm->dic[toupper(c)] = sm->dic[tolower(c)];
    }
  }
  sm->revdic = (char *)realloc(sm->revdic,(sm->size+1)*sizeof(char));
  sm->revdic[sm->size] = '\0';
  sm->mat = (float **)emalloc(sm->size*sizeof(float *));
  for(i=0;i<sm->size;i++){
    sm->mat[i] = (float *)emalloc(sm->size*sizeof(float));
  }
  /* loop over matrix body */
  for(i=0;i<sm->size;i++){
    line = fgets(line, MAXLINE, fp);
    l = 0;
    /* skip leading characters and blanks */
    while(isspace(line[l]) || isalpha(line[l]))
      l++;
    /* loop over elements in line */
    for(j=0;j<sm->size;j++){
      n = 0;
      while(!isspace(num[n++] = line[l++]))
	;
      num[--n] = '\0';
      sm->mat[i][j] = atoi(num);
      while(isspace(line[l++]))
	;
      l--;
    }
  }

  free(line);
  free(num);

  return sm;
}

/* constructDnaScoreMatrix: generate dna score matrix using the follwing
 * IUB codes with the additional provision that lower and upper case characters
 * are treated the same.
 *
 * IUB codes for nucleotides
 * Substitution is for OR'd A=1, C=2, G=4 and T=8
 *
 * A	1	A
 * B	14	TGC
 * C	2	C
 * D	13	TGA
 * G	4	G
 * H	11	TCA
 * K	12	TG
 * M	3	CA
 * N	15	TGCA
 * R	5	GA
 * S	6	GC
 * T	8	T
 * U	8	T
 * V	7	GCA
 * W	9	TA
 * X	15	TGCA
 * Y	10	TC
 */
SubstitutionMatrix *constructDnaScoreMatrix(SubstitutionMatrix *sm, int matchScore, int mismatchScore){
  int i, j;
  double p;

  sm = (SubstitutionMatrix *)emalloc(sizeof(SubstitutionMatrix));
  sm->dic = (int *)emalloc(SIZE_CHARSET*sizeof(int));
  sm->revdic = (char *)emalloc(SIZE_CHARSET*sizeof(char));
  sm->size = 16;
  sm->mat = (float **)emalloc(sm->size*sizeof(float *));
  for(i=0;i<sm->size;i++){
    sm->mat[i] = (float *)emalloc(sm->size*sizeof(float));
  }

  for(i=1; i<16; i++){
    for(j=1; j<16; j++){
      p = (float)numMatch(i,j)/(float)(numLetters(i)*numLetters(j));
        sm->mat[i][j] = round(p*(double)matchScore+(1.0-p)*(double)mismatchScore);
    }
  }
  for(i=0;i<16;i++){
    sm->mat[0][i] = 0;
    sm->mat[i][0] = 0;
  }
  for(i=0;i<SIZE_CHARSET;i++){
    sm->dic[i] = 0;
    sm->revdic[i] = 0;
  }
  
  sm->dic['A']=1;
  sm->dic['a']=1;
  sm->dic['B']=14;
  sm->dic['b']=14;
  sm->dic['C']=2;
  sm->dic['c']=2;
  sm->dic['D']=13;
  sm->dic['d']=13;
  sm->dic['G']=4;
  sm->dic['g']=4;
  sm->dic['H']=11;
  sm->dic['h']=11;
  sm->dic['K']=12;
  sm->dic['k']=12;
  sm->dic['M']=3;
  sm->dic['m']=3;
  sm->dic['N']=15;
  sm->dic['n']=15;
  sm->dic['R']=5;
  sm->dic['r']=5;
  sm->dic['S']=6;
  sm->dic['s']=6;
  sm->dic['T']=8;
  sm->dic['t']=8;
  sm->dic['U']=8;
  sm->dic['u']=8;
  sm->dic['V']=7;
  sm->dic['v']=7;
  sm->dic['W']=9;
  sm->dic['w']=9;
  sm->dic['X']=15;
  sm->dic['x']=15;
  sm->dic['Y']=10;
  sm->dic['y']=10;
  sm->revdic[1]  = 'A';
  sm->revdic[2]  = 'C';
  sm->revdic[3]  = 'M';
  sm->revdic[4]  = 'G';
  sm->revdic[5]  = 'R';
  sm->revdic[6]  = 'S';
  sm->revdic[7]  = 'V';
  sm->revdic[8]  = 'T';
  sm->revdic[9]  = 'W';
  sm->revdic[10] = 'Y';
  sm->revdic[11] = 'H';
  sm->revdic[12] = 'K';
  sm->revdic[13] = 'D';
  sm->revdic[14] = 'B';
  sm->revdic[15] = 'N';
  return sm;
}

/* int round(float x){ */
/*   double ip; */

/*   if(modf(x, &ip) >= 0.5){ */
/*     return (int) ++ip; */
/*   }else if(modf(x, &ip) <= -0.5){ */
/*     return (int) --ip; */
/*   }else{ */
/*     return (int) ip; */
/*   } */
/* } */

int numLetters(int x){
  int match = 0;

  decode(x);
  if(letters[8])
    match++;
  if(letters[4])
    match++;
  if(letters[2])
    match++;
  if(letters[1])
    match++;

  return match;
     
}

int numMatch(int x, int y){
   int match, a, b;
  
   a = min(x,y);
   b = max(x,y);
   decode(a);
   match = 0;
   
   if(b >= 8){
     if(letters[8])
       match++;
     b -= 8;
   }
   if(b >= 4){
     if(letters[4])
       match++;
     b -= 4;
   }
   if(b >= 2){
     if(letters[2])
       match++;
     b -= 2;
   }
   if(b >= 1){
     if(letters[1])
       match++;
   }
   return match;
 } 

/* decode an integer into composite nucelotides */
void decode(int n){

  if(n >= 8){
    letters[8] = 1;
    n -= 8;
  }else
    letters[8] = 0;
  if(n >= 4){
    letters[4] = 1;
    n -= 4;
  }else
    letters[4] = 0;
  if(n >= 2){
    letters[2] = 1;
    n -= 2;
  }else
    letters[2] = 0;
  if(n >= 1)
    letters[1] = 1;
  else
    letters[1] = 0;

}

/* generate dynamic programming matrix */
DpMatrix *generateDpMatrix(int m, int n, DpMatrix *dpm){
  int i, j, k, c = 0;

  dpm = (DpMatrix *)emalloc(sizeof(DpMatrix));
  dpm->mat = (DpCell ***)emalloc(m*sizeof(DpCell **));
  for(i=0; i<m; i++)
    dpm->mat[i] = (DpCell **)emalloc(n*sizeof(DpCell *));
  for(i=0;i<m;i++){
    for(j=0;j<n;j++){
      dpm->mat[i][j] = (DpCell *)emalloc(3*sizeof(DpCell));
      for(k=0;k<3;k++)
	dpm->mat[i][j][k].id = c++;
    }
  }
  return dpm;
}

int max(int a, int b){
  return (a > b) ? a : b;
}

int min(int a, int b){
  if(a<b)
    return a;
  else
    return b;
}

void freeDpm(DpMatrix *dpm){
  int m,  n, j, i;
  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;
  for(i=0;i<m;i++)
    for(j=0;j<n;j++){
      free(dpm->mat[i][j]);
    }
  for(i=0;i<m;i++)
    free(dpm->mat[i]);
  free(dpm->mat);
}

void freeAlignment(Alignment *al){
  SeqList *lp, *next;
  /* free alignment lists */
  lp = al->head1;
  for(; lp != NULL; lp = next){
    next = lp->next;
    free(lp);
  }
  lp = al->head2;
  for(; lp != NULL; lp = next){
    next = lp->next;
    free(lp);
  }
  /* free alignment structure itself */
  free(al);
}

void freeMatchList(MatchList *ml){
  MatchList *next;
  /* free list elements */
  for( ; ml != NULL; ml = next){
    next = ml->next;
    free(ml);
  }
}

void freeSubstitutionMatrix(SubstitutionMatrix *sm){
  free(sm->mat);
  free(sm->dic);
  free(sm->revdic);
  free(sm);
}

/* traceBack: generate alignment corresponding to DpMatrix; returns NULL
 * if alignment crosses path of previously returned alignment; alignment assumed
 * to be allocated */
Alignment *traceBack(DpMatrix *dpm, Alignment *al, MatrixPosition *pos, int startQuery, int startSubject, int maxError){
  SeqList *hlist = NULL, *vlist = NULL;
  int starti, startj, k, knext;
  
  starti = pos->i;
  startj = pos->j;

  al->end1 = startQuery;
  al->end2 = startSubject;
  al->length1 = strlen(dpm->hseq);
  al->length2 = strlen(dpm->vseq);


  k = pos->k;
  al->score = dpm->mat[starti][startj][k].s;

  al->numerr = 0;

  while(dpm->mat[starti][startj][k].next > -1){
    hlist = (SeqList *)emalloc(sizeof(SeqList));
    vlist = (SeqList *)emalloc(sizeof(SeqList));
    if(dpm->mat[starti][startj][k].isVisited)
       return NULL;
    dpm->mat[starti][startj][k].isVisited = 1;
    dpm->mat[starti][startj][k].s = 0;
    knext = dpm->mat[starti][startj][k].next;
    if(k == 0){
      (*hlist).c = dpm->hseq[startj-1];
      (*vlist).c = dpm->vseq[starti-1];
      starti--;
      startj--;
      startQuery--;
      startSubject--;
      if(dpm->hseq[startj] != dpm->vseq[starti])
	al->numerr++;
    }else if(k == 1){
      (*hlist).c = '-';
      (*vlist).c = dpm->vseq[starti-1];
      starti--;
      startSubject--;
      al->numerr++;
    }else{
      (*hlist).c = dpm->hseq[startj-1];
      (*vlist).c = '-';
      startj--;
      startQuery--;
      al->numerr++;
    }
    hlist->next = al->head1;
    al->head1 = hlist;
    vlist->next = al->head2;
    al->head2 = vlist;
    if(al->numerr > maxError)
      return NULL;
    if(knext < 0)
      break;
    else
      k = knext;
  }
  al->start1 = startQuery + 1;
  al->start2 = startSubject + 1;
  return al;
}

void writeDpm(DpMatrix *dpm){
  int i, j, m, n;

  m = strlen(dpm->vseq) + 1;
  n = strlen(dpm->hseq) + 1;

  printf("a: match matrix\n");
  for(i=0;i<m;i++){
    for(j=0;j<n;j++){
      printf("%.1f ",dpm->mat[i][j][0].s);
    }
    printf("\n");
  }
  printf("b: gap in horizontal sequence \n");
  for(i=0;i<m;i++){
    for(j=0;j<n;j++){
      printf("%.1f ",dpm->mat[i][j][1].s);
    }
    printf("\n");
  }
  printf("c: gap in vertical sequence \n");
  for(i=0;i<m;i++){
    for(j=0;j<n;j++){
      printf("%.1f ",dpm->mat[i][j][2].s);
    }
    printf("\n");
  }
}

void setAlignmentMode(int mode){
  alignmentMode = mode;
}
