/***** kerrorInterface.c ******************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 19:53:47 2004.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include "eprintf.h"
#include "interface.h"
#include "alignment.h"

Args getArgs(int argc, char *argv[]){
  Args args;
  int c;
  int arg;
  int input;

  args.i = NULL;
  args.j = NULL;
  args.o = NULL;
  args.O = -INT_MAX;
  args.E = -INT_MAX;
  args.A = MATCH;
  args.I = MISMATCH;
  args.m = NULL;
  args.k = 1;
  args.p = 0;
  args.a = 1;
  args.l = LINE_LENGTH;
  args.r = 0;
  args.h = 0;
  args.v = 0;
  args.s = 0;
  args.n = 0;
  args.e = 0;
  args.L = 0;
  input = 0;
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
      switch(c){
      case 'i':                           /* first input file */
	args.i = argv[++arg];
	input++;
	break;
      case 'j':                           /* second input file */
	args.j = argv[++arg];
	input++;
	break;
      case 'o':                           /* output file */
	args.o = argv[++arg];
	break;
      case 'O':                           /* gap opening score */
	args.O = atof(argv[++arg]);
	break;
      case 'E':                           /* gap extension score */
	args.E = atoi(argv[++arg]);
	break;
      case 'A':                           /* match score */
	args.A = atoi(argv[++arg]);
	break;
      case 'I':                           /* mismatch score */
	args.I = atoi(argv[++arg]);
	break;
      case 'm':                           /* external score matrix */
	args.m = argv[++arg];
	break;
      case 'p':                           /* protein sequence */
	args.p = 1;
	break;
      case 'L':                           /* print fragment List */
	args.L = 1;
	break;
      case 'l':                           /* line length */
	args.l = atoi(argv[++arg]);
	break;
      case 'r':                           /* include reverse strand */
	args.r = 1;
	break;
      case 'a':                           /* do not print alignment */
	args.a = 0;
	break;
      case 'k':                           /* maximal number of errors allowed */
	args.k = atoi(argv[++arg]);
	break;
      case 'v':
	args.v = 1;
	break;
      case 'h':                           /* print help */
	args.h = 1;
	break;
      default:
	printf("# unknown argument: %c\n",c);
	args.e = 1;
	return args;
      }
      arg++;
  }
  if(args.p && args.m == NULL){
    printf("ERROR [global]: please enter path for protein score matrix using -m\n");
    exit (-1);
  }
  if(args.p){
    if(args.O == -INT_MAX)
      args.O = GAP_OPENING_PROTEIN;
    if(args.E == -INT_MAX)
      args.E = GAP_EXTENSION_PROTEIN;
  }else{
    if(args.O == -INT_MAX)
      args.O = GAP_OPENING_DNA;
    if(args.E == -INT_MAX)
      args.E = GAP_EXTENSION_DNA;
  }
  if(input < 2)
    args.e = 1;
  return args;
}

void printUsage(char *version){
  printf("Usage: kerror -i <query.fasta> -j <sbjct.fasta> [options]\n");
  printf("Compute optimal local alignment of two molecular sequences.\n");
  printf("Options:\n");
  printf("arguments:  \n");
  printf("\t-i file containing query sequence in FASTA format\n");
  printf("\t-j file containing subject sequence in FASTA format\n");
  printf("\t[-o output file; default: stdout]\n");
  printf("\t[-k maximal number of errors (gaps + mismatches); default: 1]\n");
  printf("\t[-O gap opening; default DNA: %d; default protein: %d]\n",GAP_OPENING_DNA, GAP_OPENING_PROTEIN);
  printf("\t[-E gap extension; default DNA: %d; default protein: %d]\n",GAP_EXTENSION_DNA, GAP_EXTENSION_PROTEIN);	     
  printf("\t[-A match; default: %d]\n",MATCH);
  printf("\t[-I mismatch; default: %d]\n",MISMATCH);	
  printf("\t[-a do not print alignment(s); default: print alignment(s)]\n");		    
  printf("\t[-m external score matrix;\n");
  printf("\t    default: computed from match & mismatch parameters, assuming DNA input]\n");
  printf("\t[-r include reverse strand in search; default: forward strand only]\n");
  printf("\t[-L print fragment list]\n");						      
  printf("\t[-l line length; default: %d]\n",LINE_LENGTH);
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
}
void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Reference: Baeza-Yates, R. A. and Perlberg, C. H. (1992). Fast and practical\n");
  printf("\tapproximate string matching. Proc. 3rd Symp. on Combinatorial Pattern\n");
  printf("\tmatching. Springer Lecture Notes in Computer Science, p. 185-192.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}

