/***** localInterface.c *******************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Jan 10 22:56:41 2005.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include "eprintf.h"
#include "interface.h"
#include "alignment.h"

Args getArgs(int argc, char *argv[]){
  Args args;
  int c;
  int arg;

  args.i = NULL;
  args.j = NULL;
  args.o = NULL;
  args.O = -INT_MAX;
  args.E = -INT_MAX;
  args.A = MATCH;
  args.I = MISMATCH;
  args.m = NULL;
  args.n = 1;
  args.p = 0;
  args.s = 1;
  args.l = LINE_LENGTH;
  args.k = -1;
  args.n = 1;
  args.a = 1;
  args.h = 0;
  args.v = 0;
  args.e = 0;
  args.L = 0; /* prevent compiler warning */
  args.r = 0; /* prevent compiler warning */
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
      switch(c){
      case 'i':                           /* first input file */
	args.i = argv[++arg];
	break;
      case 'j':                           /* second input file */
	args.j = argv[++arg];
	break;
      case 'o':                           /* output file */
	args.o = argv[++arg];
	break;
      case 'O':                           /* gap opening score */
	args.O = atof(argv[++arg]);
	break;
      case 'E':                           /* gap extension score */
	args.E = atof(argv[++arg]);
	break;
      case 'A':                           /* match score */
	args.A = atof(argv[++arg]);
	break;
      case 'I':                           /* mismatch score */
	args.I = atof(argv[++arg]);
	break;
      case 's':                           /* return alignments with score >= s */
	args.s = atoi(argv[++arg]);
	break;
      case 'a':                           /* do not print alignment */
	args.a = 0;
	break;
      case 'n':                           /* number of alignmnets to return */
	args.n = atoi(argv[++arg]);
	break;
      case 'm':                           /* external score matrix */
	args.m = argv[++arg];
	break;
      case 'p':                           /* protein sequence */
	args.p = 1;
	break;
      case 'l':                           /* line length */
	args.l = atoi(argv[++arg]);
	break;
      case 'h':                           /* print help */
	args.h = 1;
	break;
      case 'v':                           /* print program information */
	args.v = 1;
	break;
      default:
	printf("# unknown argument: %c\n",c);
	args.e = 1;
	return args;
      }
      arg++;
  }
  if(args.p && args.m == NULL){
    printf("ERROR [localInterface.c]: please enter path for protein score matrix using -m\n");
    exit(-1);
  }
  if(args.p){
    if(args.O == -INT_MAX)
      args.O = GAP_OPENING_PROTEIN;
    if(args.E == -INT_MAX)
      args.E = GAP_EXTENSION_PROTEIN;
  }else{
    if(args.O == -INT_MAX)
      args.O = GAP_OPENING_DNA;
    if(args.E == -INT_MAX)
      args.E = GAP_EXTENSION_DNA;
  }
  if(args.i == NULL)
    args.e = 1;
  return args;
}


void printUsage(char *version){
  printf("Usage: lal -i <query.fasta> [options]\n");
  printf("Compute optimal local alignment of two molecular sequences.\n");
  printf("Options:\n");
  printf("\t-i file containing query sequence in FASTA format\n");
  printf("\t[-j file containing subject sequence in FASTA format; default: stdin]\n");
  printf("\t[-o output file; default: stdout]\n");
  printf("\t[-O gap opening; default DNA: %d; default protein: %d]\n",GAP_OPENING_DNA, GAP_OPENING_PROTEIN);
  printf("\t[-E gap extension; default DNA: %d; default protein: %d]\n",GAP_EXTENSION_DNA, GAP_EXTENSION_PROTEIN);   
  printf("\t[-A match; default: %d]\n",MATCH);
  printf("\t[-I mismatch; default: %d]\n",MISMATCH);	
  printf("\t[-a do not print alignment(s); default: print alignment(s)]\n");	
  printf("\t[-m external score matrix - necessary for protein sequences;\n");
  printf("\t    default: computed from match & mismatch parameters, assuming DNA input]\n");
  printf("\t[-n number of best alignments to return; default: 1]\n");
  printf("\t[-s return alignments with score >= s; default: 1]\n");
  printf("\t[-p protein sequence; default: nucleotide sequence]\n");
  printf("\t[-l line length; default: %d]\n",LINE_LENGTH);
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
}
void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Reference: Gusfield, D. (1997). Algorithms on Strings, Trees, and Sequences. CUP, ch. 11.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
