/***** overlap.c **************************************************
 * Description: Overlap alignment of protein or DNA sequences.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 19:53:38 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "eprintf.h"
#include "interface.h"
#include "sequenceData.h"
#include "alignment.h"
#include "stringUtil.h"

int main(int argc, char *argv[]){
  char *version;
  Args args; 
  Sequence *s1 = NULL, *s2 = NULL;
  Alignment *al;
  SubstitutionMatrix *sm = NULL;
  FILE *fp;

  setprogname2("oal");
  setAlignmentMode(OVERLAP);

  version = VERSION;
  args = getArgs(argc, argv);
  
  if(args.h == 1){
    printUsage(version);
    return 0; 
  }else if(args.v == 1){
    printSplash(version);
    return 0;
  }else if(args.e == 1){
    printUsage(version);
    return -1;
  }

  fp = efopen(args.i,"r");
  s1 = getNextSequence(fp);
  resetSequenceReader();
  fclose(fp);
  if(args.j)
    fp = efopen(args.j,"r");
  else
    fp = stdin;
  s2 = getNextSequence(fp);
  fclose(fp);

  if(args.m == NULL){
    sm = constructDnaScoreMatrix(sm, args.A, args.I);
  }else{
    fp = efopen(args.m, "r");
    sm = readSubstitutionMatrix(sm, fp);
  }
  al = overlap(s1->seq,s2->seq,args.O,args.E,sm);
  al->name1 = strdup2(s1->id);
  al->name2 = strdup2(s2->id);
  if(args.o == NULL)
    fp = stdout;
  else
    fp = efopen(args.o,"w");
  if(args.p)
    al->strand = NULL;
  else
    al->strand = "Plus / Plus";
  printAlignment(al,fp,sm,args);
  
  return 0;
}
