/***** overlapAlignment.c ******************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 12:07:53 2004.
 *****************************************************************/
#include "string.h"
#include "eprintf.h"
#include "interface.h"
#include "alignment.h"
#include "limits.h"

DpMatrix *initializeOverlapDpm(DpMatrix *dpm);
MatrixPosition *findOverlapMax(DpMatrix *dpm);

/* optimal overlap alignment of two sequences */
Alignment *overlap(char *seq1, char *seq2, float open, float extend, SubstitutionMatrix *sm){
  int a, b, c, i, j = 0, m, n, x;

  open += extend;       /* gap of length 1 has weight even if gap opening is set to zero */
  DpMatrix *dpm = NULL;
  Alignment *al = NULL;
  SeqList *lp;
  MatrixPosition *pos;

  n = strlen(seq1) + 1;
  m = strlen(seq2) + 1;
  
  dpm = generateDpMatrix(m, n, dpm);
  dpm->hseq = seq1;
  dpm->vseq = seq2;
  dpm = initializeOverlapDpm(dpm);
  /* fill in remainder of matrix */
  for(i=1; i<m; i++){
    for(j=1; j<n; j++){
      a = dpm->mat[i-1][j][0].s + open;
      b = dpm->mat[i-1][j][1].s + extend;
      if(a > b){
	dpm->mat[i][j][1].s = a;
	dpm->mat[i][j][1].next = 0;
      }else{
	dpm->mat[i][j][1].s = b;
	dpm->mat[i][j][1].next = 1;
      }
      /* fill in c */
      a = dpm->mat[i][j-1][0].s + open;
      c = dpm->mat[i][j-1][2].s + extend;
      if(a > c){
	dpm->mat[i][j][2].s = a;
	dpm->mat[i][j][2].next = 0;
      }else{
	dpm->mat[i][j][2].s = c;
	dpm->mat[i][j][2].next = 2;
      }
      /* fill in a */
      a = dpm->mat[i-1][j-1][0].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      b = dpm->mat[i-1][j-1][1].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      c = dpm->mat[i-1][j-1][2].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      x = max(a,max(b,c));
      if(a == x){
	dpm->mat[i][j][0].s = a;
	dpm->mat[i][j][0].next = 0;
      }else if(b == x){
	dpm->mat[i][j][0].s = b;
	dpm->mat[i][j][0].next = 1;
      }else if(c == x){
	dpm->mat[i][j][0].s = c;
	dpm->mat[i][j][0].next = 2;
      }
    }
  }
  //writeDpm(dpm);
  al = (Alignment *)emalloc(sizeof(Alignment));
  al->head1 = NULL;
  al->head2 = NULL;
  pos = findOverlapMax(dpm);
  for(i=m-1;i>pos->i;i--){
    lp = (SeqList *)emalloc(sizeof(SeqList));
    lp->c = dpm->vseq[i-1];
    lp->next = al->head2;
    al->head2 = lp;
    lp = (SeqList *)emalloc(sizeof(SeqList));
    lp->c = '-';
    lp->next = al->head1;
    al->head1 = lp;
  }
  for(j=n-1;j>pos->j;j--){
    lp = (SeqList *)emalloc(sizeof(SeqList));
    lp->c = dpm->hseq[j-1];
    lp->next = al->head1;
    al->head1 = lp;
    lp = (SeqList *)emalloc(sizeof(SeqList));
    lp->c = '-';
    lp->next = al->head2;
    al->head2 = lp;
  }
  al = traceBack(dpm,al,pos,pos->j,pos->i,INT_MAX);
  return al;
}

MatrixPosition *findOverlapMax(DpMatrix *dpm){
  int i, j = 0, k, m, n;
  int maxVal = INT_MIN;
  MatrixPosition *pos;

  pos = (MatrixPosition *)emalloc(sizeof(MatrixPosition));
  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;
  /* check last column */
  pos->j = n-1;
  for(i=0;i<m;i++){
    for(k=0;k<3;k++){
      if(dpm->mat[i][n-1][k].s > maxVal){
	pos->i = i;
	pos->k = k;
	maxVal = dpm->mat[i][n-1][k].s;
      }
    }
  }
  /* check last row */
  for(j=0;j<n;j++){
    for(k=0;k<3;k++){
      if(dpm->mat[m-1][j][k].s > maxVal){
	pos->i = m-1;
	pos->j = j;
	pos->k = k;
	maxVal = dpm->mat[m-1][j][k].s;
      }
    }
  }
  return pos;
}

DpMatrix *initializeOverlapDpm(DpMatrix *dpm){
  int i, m, n;

  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;
 
  dpm->mat[0][0][0].s = 0;
  dpm->mat[0][0][1].s = 0;
  dpm->mat[0][0][2].s = 0;
  dpm->mat[0][0][0].next = -1;
  dpm->mat[0][0][1].next = -1;
  dpm->mat[0][0][2].next = -1;
  /* initialize first column */
  for(i=1;i<m;i++){
    dpm->mat[i][0][0].s = INT_MIN/2;
    dpm->mat[i][0][1].s = 0;
    dpm->mat[i][0][2].s = INT_MIN/2;
    dpm->mat[i][0][0].next = -1;
    dpm->mat[i][0][1].next = 1;
    dpm->mat[i][0][2].next = -1;
  }
  /* initialize first row */
  for(i=1;i<n;i++){
    dpm->mat[0][i][0].s = INT_MIN/2;
    dpm->mat[0][i][1].s = INT_MIN/2;
    dpm->mat[0][i][2].s = 0;
    dpm->mat[0][i][0].next = -1;
    dpm->mat[0][i][1].next = -1;
    dpm->mat[0][i][2].next = 2;
  }
  return dpm;
}
